/*	$NetBSD$	*/

/*-
 * Copyright (c) 2010 SHIMIZU Ryo <ryo@nerv.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD$");

#include <sys/param.h>
#include <sys/bus.h>
#include <sys/device.h>
#include <uvm/uvm_extern.h>

#include <arm/imx/imx51reg.h>
#include <arm/imx/imx51var.h>

void
imx51_reset()
{
	printf("imx51_reset (not implemented yet)\n");
}

void
imx51_initialize()
{
}

struct imx51_devtable imx51_devtable[] = {
	{
		.paddr = IMX51_UART0_BASE,
		.vaddr = IMX51_UART0_BASE_VADDR,
		.size = IMX51_UART_SIZE
	},
	{
		.paddr = IMX51_UART1_BASE,
		.vaddr = IMX51_UART1_BASE_VADDR,
		.size = IMX51_UART_SIZE
	},
	{
		.paddr = IMX51_UART2_BASE,
		.vaddr = IMX51_UART2_BASE_VADDR,
		.size = IMX51_UART_SIZE
	},
	{
		.paddr = IMX51_TZIC_BASE,
		.vaddr = IMX51_TZIC_BASE_VADDR,
		.size = IMX51_TZIC_SIZE
	}
};

int
imx51_build_devmap(struct pmap_devmap *devmap, int ndevmap)
{
	int i;

	for (i = 0; i < __arraycount(imx51_devtable); i++) {
		if (i >= ndevmap)
			return -1;

		devmap[i].pd_pa = imx51_devtable[i].paddr;
		devmap[i].pd_va = imx51_devtable[i].vaddr;
		devmap[i].pd_size = imx51_devtable[i].size;
		devmap[i].pd_prot = VM_PROT_READ | VM_PROT_WRITE;
		devmap[i].pd_cache = PTE_NOCACHE;
	}

	/* clear last element */
	memset(&devmap[i], 0, sizeof(devmap[i]));

	return i;
}

