/*	$NetBSD$ */

/*-
 * Copyright (c) 2010 SHIMIZU Ryo <ryo@nerv.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD$");

#include <sys/types.h>
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/kernel.h>
#include <sys/atomic.h>
#include <sys/time.h>
#include <sys/timetc.h>
#include <sys/device.h>

#include <dev/clock_subr.h>

#include <machine/bus.h>
#include <machine/intr.h>

#include <arm/cpufunc.h>

#include <arm/imx/imx51reg.h>
#include <arm/imx/imx51var.h>

/* specified stathz. if set 0, statclock shared with hardclock */
#ifndef STATHZ
#define STATHZ	64
#endif

struct epit_softc {
	device_t sc_dev;
	bus_addr_t sc_addr;
	bus_space_tag_t sc_iot;
	bus_space_handle_t sc_ioh;
	void *sc_ih;
	uint32_t sc_timer_clk_per_hz;
};

static struct epit_softc *clock_sc;

static int epit_match(device_t, struct cfdata *, void *);
static void epit_attach(device_t, device_t, void *);
static u_int epit_get_timecount(struct timecounter *);
static void epit_timer_set(struct epit_softc *sc, uint32_t tick);
static int epit_timer_intr(void *);	/* interrupt handler */

static struct timecounter epit_timecounter = {
	.tc_get_timecount = epit_get_timecount,
	.tc_poll_pps = NULL,
	.tc_counter_mask = ~0U,
	.tc_frequency = IMX51_CLOCK_FREQ,
	.tc_name = "epit",
	.tc_quality = 100,
	.tc_priv = NULL,
	.tc_next = NULL
};


CFATTACH_DECL_NEW(epit, sizeof(struct epit_softc),
    epit_match, epit_attach, NULL, NULL);

/* ARGSUSED */
static int
epit_match(device_t parent __unused, struct cfdata *match __unused,
    void *aux)
{
	struct axi_attach_args *aa;

	aa = aux;

	aa->aa_size = IMX51_EPIT_SIZE;
	return 1;
}

/* ARGSUSED */
static void
epit_attach(device_t parent __unused, device_t self, void *aux)
{
	struct epit_softc *sc;
	struct axi_attach_args *aa;

	aprint_normal(": Enhanced Periodic Interrupt Timer\n");
	aprint_naive("\n");

	aa = aux;
	sc = device_private(self);
	sc->sc_dev = self;
	sc->sc_iot = aa->aa_iot;
	sc->sc_addr = aa->aa_addr;

	if (bus_space_map(sc->sc_iot, sc->sc_addr, aa->aa_size, 0, &sc->sc_ioh))
	    panic("%s: Cannot map registers", device_xname(self));

	/*
	 * clock_sc would be referenced from cpu_initclocks(),
	 * setstatclockrate(), interrupt routines, etc.
	 */
	clock_sc = sc;
}

void
setstatclockrate(int newhz)
{
	struct epit_softc *sc;

	sc = clock_sc;
}

void
cpu_initclocks(void)
{
	struct epit_softc *sc;

	sc = clock_sc;
	if (sc == NULL)
		panic("cpu_initclocks: epit not attached");

	profhz = stathz = STATHZ;

	/*
	 * reset timer counter
	 */

	// notyet


	/*
	 * enable interrupts
	 */

	// notyet
//	sc->sc_ih1 = imx51_intr_establish(IRQ_EPIT1, IPL_CLOCK,
//	    epit_timer_intr, NULL);
//	if (sc->sc_ih1 == NULL)
//		panic("cpu_initclocks: unable to register timer1 interrupt");



	tc_init(&epit_timecounter);
}

/* ARGSUSED */
static u_int
epit_get_timecount(struct timecounter *tc __unused)
{
	static u_int xxx;

	return ++xxx;
}

void
delay(unsigned int us)
{
	struct epit_softc *sc;
	uint32_t prev, now, reload;
	int32_t left;

	sc = clock_sc;
//	reload = readclock();
	left = (long long)us * epit_timecounter.tc_frequency / 1000000;

//	prev = readclock();
	do {
//		now = readclock();
		if (now > prev)
			left -= prev + reload - now;
		else
			left -= prev - now;
		prev = now;
	} while (left > 0);
}
