/*	$NetBSD$	*/

/*-
 * Copyright (c) 2010 SHIMIZU Ryo <ryo@nerv.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD$");

#define _INTR_PRIVATE	/* for arm/pic/picvar.h */

#include "locators.h"

#include <sys/param.h>
#include <sys/evcnt.h>
#include <sys/device.h>
#include <machine/bus.h>
#include <arm/imx/imx51reg.h>
#include <arm/imx/imx51var.h>

static int tzic_match(device_t, cfdata_t, void *);
static void tzic_attach(device_t, device_t, void *);

/* for arm/pic */
static void tzic_unblock_irqs(struct pic_softc *, size_t, uint32_t);
static void tzic_block_irqs(struct pic_softc *, size_t, uint32_t);
static int tzic_find_pending_irqs(struct pic_softc *);
static void tzic_establish_irq(struct pic_softc *, struct intrsource *);
static void tzic_source_name(struct pic_softc *, int, char *, size_t);

struct tzic_softc {
	device_t sc_dev;
	bus_addr_t sc_addr;
	bus_size_t sc_size;
	bus_space_tag_t sc_iot;
	bus_space_handle_t sc_ioh;
	struct pic_softc sc_pic;
};

const struct pic_ops tzic_pic_ops = {
	.pic_unblock_irqs = tzic_unblock_irqs,
	.pic_block_irqs = tzic_block_irqs,
	.pic_find_pending_irqs = tzic_find_pending_irqs,
	.pic_establish_irq = tzic_establish_irq,
	.pic_source_name = tzic_source_name
};

static struct tzic_softc *tzic_sc;
static const char * const tzic_intr_source_names[] = TZIC_INTR_SOURCE_NAMES;

CFATTACH_DECL_NEW(tzic, sizeof(struct tzic_softc),
    tzic_match, tzic_attach, NULL, NULL);

/* ARGSUSED */
int
tzic_match(device_t parent, cfdata_t self, void *aux)
{
	struct axi_attach_args *aa;

	aa = aux;

	if (aa->aa_addr != IMX51_TZIC_BASE)
		return 0;
	aa->aa_size = IMX51_TZIC_SIZE;

	return 1;
}

void
tzic_attach(device_t parent, device_t self, void *aux)
{
	struct tzic_softc *sc;
	struct axi_attach_args *aa;

	aprint_normal(": TrustZone Interrupt Controller\n");
	aprint_naive("\n");

	aa = aux;
	tzic_sc = sc = device_private(self);
	sc->sc_dev = self;
	sc->sc_iot = aa->aa_iot;
	sc->sc_addr = aa->aa_addr;

	if (bus_space_map(sc->sc_iot, sc->sc_addr, sc->sc_size, 0, &sc->sc_ioh))
		panic("%s: Cannot map registers", device_xname(self));

	sc->sc_pic.pic_ops = &tzic_pic_ops;
	sc->sc_pic.pic_maxsources = IMX51_TZIC_INTNUM;
	strlcpy(sc->sc_pic.pic_name, device_xname(self),
	    sizeof(sc->sc_pic.pic_name));

	pic_add(&sc->sc_pic, 0);
}


void
tzic_unblock_irqs(struct pic_softc *pic, size_t irq_base, uint32_t irq_mask)
{
	/* XXX: notyet */
}

void
tzic_block_irqs(struct pic_softc *pic, size_t irq_base, uint32_t irq_mask)
{
	/* XXX: notyet */
}

static int
tzic_find_pending_irqs(struct pic_softc *pic)
{
	/* XXX: notyet */

	return 0;
}

void
tzic_establish_irq(struct pic_softc *pic, struct intrsource *is)
{
	/* XXX: notyet */
}

void
tzic_source_name(struct pic_softc *pic, int irq, char *buf, size_t len)
{
	strlcpy(buf, tzic_intr_source_names[irq], len);
}

void
imx51_irq_handler(void *frame)
{
//	int32_t saved_nimask;
//	int32_t irq;
//	int ipl, newipl, oldipl;

	/* XXX: notyet */
}

void
imx51_intr_init()
{
}
